#ifndef DUALLISTALGOR_H_JRI304
#define DUALLISTALGOR_H_JRI304

#include "TestAlgor.h"
#include "Particle.h"
#include "Util.h"
#include <string>
#include <boost/timer.hpp>

class DualListAlgor : public TestAlgor {
  virtual ~DualListAlgor() {}

  virtual std::string getName() const {
    return "Dual List";
  }

  virtual double operator()( int numLoops, float dt ) const {
    typedef std::list< Particle* > PList;
    typedef PList::iterator PListIter;

    PList l;
    PList dead;

    boost::timer timer;

    //Run our "game loop"
    for ( int c = 0; c < numLoops; ++c ) {
      if ( !dead.empty() ) {
        //Elements in the back most likely in CPU cache
        Particle* newP = dead.back();
        dead.pop_back();
        newP->reinit(
          Vector3D( getRandF(), getRandF(), getRandF() ),
          Vector3D( getRandF(), getRandF(), getRandF() ),
          getRandI(),
          getRandF(),
          getRandF() );
        l.push_back( newP );
      } else {
        l.push_back( new Particle(
          Vector3D( getRandF(), getRandF(), getRandF() ),
          Vector3D( getRandF(), getRandF(), getRandF() ),
          getRandI(),
          getRandF(),
          getRandF() ) );
      }

      PListIter iter = l.begin();
      while ( iter != l.end() ) {
        if ( (*iter)->update2( dt ) ) {
          dead.push_back( *iter );
          iter = l.erase( iter );
        } else
          ++iter;
      }
    }

    return timer.elapsed();

    std::for_each( l.begin(), l.end(), Deleter() );
    std::for_each( dead.begin(), dead.end(), Deleter() );
  }
};

#endif //DUALLISTALGOR_H_JRI304
