#ifndef PARTICLE_H_MNF499
#define PARTICLE_H_MNF499

/**
 * A "typical" vector class.
 */
class Vector3D {
public:
  Vector3D() : x(0.0f), y(0.0f), z(0.0f) {
  }

  Vector3D( float x, float y, float z ) : x(x), y(y), z(z) {
  }

  Vector3D( const Vector3D& o ) : x(o.x), y(o.y), z(o.z) {
  }

  void multAndAdd( const Vector3D& other, const float scalar ) {
    x += other.x * scalar;
    y += other.y * scalar;
    z += other.z * scalar;
  }

private:
  float x;
  float y;
  float z;
};

/**
 * A "typical" particle class, with a datasize that would be typical of a particle.
 */
class Particle {
public:
  /**
   * Default constructor makes an arbitrary object whose isDead method returns
   * true, and no other methods except the destructor are valid calls.
   */
  Particle() : life( -1.0f ) {
  }

  Particle( const Vector3D& pos, const Vector3D& velo, int color, float size, float life ) :
      pos(pos), velo(velo), color(color), size(size), life(life) {
  }

  Particle( const Particle& o ) :
      pos(o.pos), velo(o.velo), color(o.color), size(o.size), life(o.life) {
  }

  void reinit( const Vector3D& pos, const Vector3D& velo, int color, float size, float life ) {
    this->pos = pos;
    this->velo = velo;
    this->color = color;
    this->size = size;
    this->life = life;
  }

  /**
   * Update used in the ParticleManager.
   */
  void update1( float dt ) {
    if ( !isDead() ) {
      life -= dt;
      pos.multAndAdd( velo, dt ); //Euler integration
    }
  }

  /**
   * Update method used in the lists.
   */
  bool update2( float dt ) {
    life -= dt;
    pos.multAndAdd( velo, dt ); //Euler integration
    return isDead();
  }

  float getLife() const {
    return life;
  }

  bool isDead() const {
    return life <= 0.0f;
  }

  void draw() {
    if ( !isDead() ) {
    }
  }

protected:
  Vector3D pos;
  Vector3D velo;

  int color;

  float size;

  float life;
};

#endif //PARTICLE_H_MNF499
