#ifndef VECTORALGOR_H_AWK404
#define VECTORALGOR_H_AWK404

#include "TestAlgor.h"
#include "Particle.h"
#include <boost/timer.hpp>
#include <vector>

/**
 * The VectorAlgor was contributed by Julien Cugniere.  It uses a packet array
 * with a copy-on-death technique.  The objects move in memory, so it is not
 * appropriate if pointers are kept to the elements.  This algorithm is
 * identical to FlatArrayAllocAlgor except that it uses a dynamically-sized
 * std::vector.
 *
 * @author Julien Cugniere
 */
class VectorAlgor : public TestAlgor {
public:
  virtual ~VectorAlgor() {}
  
  virtual std::string getName() const {
    return "VectorAlgor";
  }
  
  virtual double operator()( int numLoops, float dt ) const {
    std::vector<Particle> particles;
    boost::timer timer;
    
    // Run our "game loop"
    for ( int c = 0; c < numLoops; ++c ) {
      particles.push_back( Particle(
        Vector3D( getRandF(), getRandF(), getRandF() ),
        Vector3D( getRandF(), getRandF(), getRandF() ),
        getRandI(),
        getRandF(),
        getRandF() ) );
      
      std::vector<Particle>::iterator i = particles.begin();
      while ( i != particles.end() ) {
        if ( i->update2( dt ) ) {
          *i = particles.back();
          particles.pop_back();
        } else
          ++i;
      }
    }
    
    return timer.elapsed();
  }
};

#endif //VECTORALGOR_H_AWK404
