#include <iostream>
#include <ctime>
#include <cassert>
#include <functional>
#include <algorithm>
#include <list>
#include <sstream>
#include <string>
#include <vector>
#include <limits>

#include <boost/timer.hpp>

#include "Particle.h"
#include "TestAlgor.h"
#include "Util.h"
#include "ParticleManager.h"
#include "VectorAlgor.h"
#include "FlatArrayAllocAlgor.h"
#include "BinListAlgor.h"
#include "ParticleManagerAlgor.h"
#include "PointerListAlgor.h"
#include "DualListAlgor.h"

typedef std::vector<int> IntList;
typedef IntList::size_type IntListIdx;

//This is used for the Objects At End line.
//It is only filled if ObjectListAlgor is run.
IntList cheatList;
#include "ObjectListAlgor.h"

#ifdef _MSC_VER
//I'm amazed the boost headers cause these warnings?
#pragma warning (disable: 4267 4244)
#endif
#include <boost/format.hpp>

using namespace std;

int main( int argc, char* argv[] ) {
  srand( (unsigned int)time(0) );

  typedef std::vector<TestAlgor*> AlgorsList;
  typedef AlgorsList::const_iterator AlgorsListCIter;

  AlgorsList algors;
  algors.push_back( new ParticleManagerAlgor( 500 ) );
  algors.push_back( new ParticleManagerAlgor( 2000 ) );
  algors.push_back( new PointerListAlgor() );
  algors.push_back( new ObjectListAlgor() );
  algors.push_back( new DualListAlgor() );
  algors.push_back( new BinListAlgor() );
  algors.push_back( new VectorAlgor() );
  algors.push_back( new FlatArrayAllocAlgor() );

  IntList numLoops;
  numLoops.push_back(  10000 );
  numLoops.push_back(  20000 );
  numLoops.push_back(  50000 );
  numLoops.push_back( 100000 );

  typedef std::vector<float> FloatList;
  typedef FloatList::size_type FloatListIdx;

  FloatList dts;
  dts.push_back( 0.1f );
  dts.push_back( 0.01f );
  dts.push_back( 0.005f );
  dts.push_back( 0.001f );

  const std::string delim = ", ";

  cout << "All times are in seconds." << endl;
  for ( IntListIdx y = 0; y < numLoops.size(); ++y ) {
    //Print Loop size line
    cout << numLoops[y] << " Loops " << endl;

    //Format into a nice table.  Modify this if dts list changes.
    boost::format row( "%1%" + delim + "%|25t|%2%" + delim + "%|35t|%3%" + delim + "%|45t|%4%" + delim + "%|55t|%5%" );
    assert( dts.size() == 4);

    //Print DT field headers
    row % ""; //first column empty
    for ( FloatListIdx x = 0; x < dts.size(); ++x ) {
      row % dts[x];
    }
    cout << row << endl;

    //For each algorithm, run the DTs for this loops case
    for ( AlgorsListCIter iter = algors.begin(); iter != algors.end(); ++iter ) {
      const TestAlgor& algor = **iter;
      row % algor.getName();

      for ( FloatListIdx dt = 0; dt < dts.size(); ++dt ) {
        row % algor( numLoops[y], dts[dt] );
      }
      cout << row << endl;
    }

    //Print objects at end line
    row % "Objects At End";
    for ( IntListIdx i = 0; i < cheatList.size(); ++i )
      row % cheatList[i];
    cout << row << endl;
    cheatList.clear();
    cout << endl;
  }

  //Delete all of the algorithms we created
  for_each( algors.begin(), algors.end(), Deleter() );

  return 0;
}
